import os

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class DBHistoryMigration(MigrationABC):
    name = "1.0.0_DBHistoryMigration"
    prio = 1

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def _exec(self, file: str):
        path = f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts"
        sql = open(f"{path}/{file}").read()

        for statement in sql.split("\n\n"):
            self._cursor.execute(statement + ";")

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._exec("api_keys.sql")
        self._exec("auth_users.sql")
        self._exec("auth_user_users_relation.sql")
        self._exec("auto_role_rules.sql")
        self._exec("auto_roles.sql")
        self._exec("clients.sql")
        self._exec("game_servers.sql")
        self._exec("known_users.sql")
        self._exec("levels.sql")
        self._exec("servers.sql")
        self._exec("user_game_idents.sql")
        self._exec("user_joined_game_servers.sql")
        self._exec("user_joined_servers.sql")
        self._exec("user_joined_voice_channel.sql")
        self._exec("user_message_count_per_hour.sql")
        self._exec("users.sql")
        self._exec("user_warnings.sql")

        self._logger.debug(__name__, "Finished history upgrade")

    def downgrade(self):
        self._cursor.execute("DROP TABLE `ApiKeysHistory`;")
        self._cursor.execute("DROP TABLE `AuthUsersHistory`;")
        self._cursor.execute("DROP TABLE `AuthUserUsersRelationsHistory`;")
        self._cursor.execute("DROP TABLE `AutoRoleRulesHistory`;")
        self._cursor.execute("DROP TABLE `AutoRolesHistory`;")
        self._cursor.execute("DROP TABLE `ClientsHistory`;")
        self._cursor.execute("DROP TABLE `GameServersHistory`;")
        self._cursor.execute("DROP TABLE `KnownUsersHistory`;")
        self._cursor.execute("DROP TABLE `LevelsHistory`;")
        self._cursor.execute("DROP TABLE `ServersHistory`;")
        self._cursor.execute("DROP TABLE `UserGameIdentsHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedGameServerHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedServersHistory`;")
        self._cursor.execute("DROP TABLE `UserJoinedVoiceChannelHistory`;")
        self._cursor.execute("DROP TABLE `UserMessageCountPerHourHistory`;")
        self._cursor.execute("DROP TABLE `UsersHistory`;")
        self._cursor.execute("DROP TABLE `UserWarningsHistory`;")
