from datetime import datetime
from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.user_message_count_per_hour_repository_abc import (
    UserMessageCountPerHourRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_message_count_per_hour import UserMessageCountPerHour


class UserMessageCountPerHourRepositoryService(UserMessageCountPerHourRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        users: UserRepositoryABC,
    ):
        UserMessageCountPerHourRepositoryABC.__init__(self)

        self._logger = logger
        self._context = db_context
        self._users = users

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and "NULL" in value:
            return None

        return value

    def _from_result(self, result: tuple) -> UserMessageCountPerHour:
        return UserMessageCountPerHour(
            self._get_value_from_result(result[1]),
            self._get_value_from_result(result[2]),
            self._get_value_from_result(result[3]),
            self._users.get_user_by_id(self._get_value_from_result(result[4])),
            self._get_value_from_result(result[5]),
            self._get_value_from_result(result[6]),
            id=self._get_value_from_result(result[0]),
        )

    def get_user_message_count_per_hours(self) -> List[UserMessageCountPerHour]:
        umcphs = List(UserMessageCountPerHour)
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserMessageCountPerHour.get_select_all_string()}",
        )
        results = self._context.select(UserMessageCountPerHour.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user message count per hour with id {result[0]}")
            umcphs.append(self._from_result(result))

        return umcphs

    def find_user_message_count_per_hour_by_user_id(self, user_id: int) -> List[Optional[UserMessageCountPerHour]]:
        umcphs = List(UserMessageCountPerHour)
        sql = UserMessageCountPerHour.get_select_by_user_id_string(user_id)
        self._logger.trace(__name__, f"Send SQL command: {sql}")
        results = self._context.select(sql)
        if results is None or len(results) == 0:
            return umcphs

        for result in results:
            self._logger.trace(__name__, f"Get user message count per hour with id {result[0]}")
            umcphs.append(self._from_result(result))

        return umcphs

    def get_user_message_count_per_hour_by_user_id_and_date(
        self, user_id: int, date: datetime
    ) -> UserMessageCountPerHour:
        sql = UserMessageCountPerHour.get_select_by_user_id_and_date_string(user_id, date)
        self._logger.trace(__name__, f"Send SQL command: {sql}")
        result = self._context.select(sql)[0]
        return self._from_result(result)

    def find_user_message_count_per_hour_by_user_id_and_date(
        self, user_id: int, date: datetime
    ) -> Optional[UserMessageCountPerHour]:
        sql = UserMessageCountPerHour.get_select_by_user_id_and_date_string(user_id, date)
        self._logger.trace(__name__, f"Send SQL command: {sql}")
        result = self._context.select(sql)
        if result is None or len(result) == 0:
            return None

        return self._from_result(result[0])

    def add_user_message_count_per_hour(self, umcph: UserMessageCountPerHour):
        self._logger.trace(__name__, f"Send SQL command: {umcph.insert_string}")
        self._context.cursor.execute(umcph.insert_string)

    def update_user_message_count_per_hour(self, umcph: UserMessageCountPerHour):
        self._logger.trace(__name__, f"Send SQL command: {umcph.udpate_string}")
        self._context.cursor.execute(umcph.udpate_string)

    def delete_user_message_count_per_hour(self, umcph: UserMessageCountPerHour):
        self._logger.trace(__name__, f"Send SQL command: {umcph.delete_string}")
        self._context.cursor.execute(umcph.delete_string)

    def delete_user_message_count_per_hour_by_user_id(self, user_id: int):
        sql = UserMessageCountPerHour.delete_by_user_id_string(user_id)
        self._logger.trace(__name__, f"Send SQL command: {sql}")
        self._context.cursor.execute(sql)
