import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_data.model.server_config import ServerConfig
from modules.level.service.level_service import LevelService


class LevelOnVoiceStateUpdateEvent(OnVoiceStateUpdateABC):
    def __init__(self, config: ConfigurationABC, logger: LoggerABC, level: LevelService):
        OnVoiceStateUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._level = level

        self._logger.info(__name__, f"Module {type(self)} loaded")

    @EventChecks.check_is_ready()
    async def on_voice_state_update(
        self,
        member: discord.Member,
        before: discord.VoiceState,
        after: discord.VoiceState,
    ):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(server_config.feature_flags, FeatureFlagsEnum.level_module):
            return

        await self._level.check_level(member)
