import { NgModule } from "@angular/core";
import { RouterModule, Routes } from "@angular/router";
import { ServerDashboardComponent } from "./server-dashboard/server-dashboard.component";
import { ProfileComponent } from "./profile/profile.component";
import { MembersComponent } from "./members/members.component";
import { AuthGuard } from "../../shared/guards/auth/auth.guard";
import { MemberRoles } from "../../../models/auth/auth-user.dto";

const routes: Routes = [
  { path: "", component: ServerDashboardComponent },
  { path: "members", component: MembersComponent, canActivate: [AuthGuard], data: { memberRole: MemberRoles.Moderator } },
  { path: "members/:memberId", component: ProfileComponent },
  {
    path: "auto-roles",
    loadChildren: () => import("./auto-role/auto-role.module").then(m => m.AutoRoleModule),
    canActivate: [AuthGuard],
    data: { memberRole: MemberRoles.Moderator }
  },
  { path: "levels", loadChildren: () => import("./levels/levels.module").then(m => m.LevelsModule), canActivate: [AuthGuard], data: { memberRole: MemberRoles.Moderator } },
  { path: "achievements", loadChildren: () => import("./achievements/achievements.module").then(m => m.AchievementsModule), data: { memberRole: MemberRoles.Moderator } },
  { path: "config", loadChildren: () => import("./config/config.module").then(m => m.ConfigModule), data: { memberRole: MemberRoles.Admin } }
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class ServerRoutingModule {
}
